function [peakparams, PSD_signal] = findSignal(PSD_input_1D, PSD_background_1D, frequencies_polar, lowestfreq, highestfreq, gaussianguess)

% Copyright 2015 Kimmo Kartasalo
% Tampere University of Technology, Tampere, Finland
% http://www.tut.fi/cytospectre
% Email: kimmo.kartasalo@tut.fi
% 
% This file is part of CytoSpectre.
% 
% CytoSpectre is free software: you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published by
% the Free Software Foundation, either version 3 of the License, or
% (at your option) any later version.
% 
% CytoSpectre is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
% 
% You should have received a copy of the GNU General Public License
% along with CytoSpectre.  If not, see <http://www.gnu.org/licenses/>.

% Subtract background.
PSD_signal = PSD_input_1D - PSD_background_1D;
% Set signal at forbidden frequencies to zero.
PSD_signal(frequencies_polar < lowestfreq) = 0;
PSD_signal(frequencies_polar > highestfreq) = 0;
% Force negative values to zero.
PSD_signal(PSD_signal < 0) = 0;

% If the guess is empty, use the minimum and maximum frequencies to select a search window.
if isempty(gaussianguess{1})
    minpeakloc = find(frequencies_polar < lowestfreq,1,'last') + 1;
    maxpeakloc = find(frequencies_polar > highestfreq,1,'first') - 1;
% Otherwise use the given guess.
else
    minpeakloc = find(frequencies_polar < gaussianguess{1},1,'last') + 1;
    maxpeakloc = find(frequencies_polar > gaussianguess{2},1,'first') - 1;
end
% Check validity.
if isempty(minpeakloc)
    minpeakloc = 1;
end
if isempty(maxpeakloc)
    maxpeakloc = length(frequencies_polar);
end

% If the search window limits were given by the user or the search window
% only contains zeros, use the search window limits as estimates of peak
% edge locations.
if ~isempty(gaussianguess{1}) || all(PSD_signal(minpeakloc:maxpeakloc) == 0)
    peakedge(1) = frequencies_polar(minpeakloc);
    peakedge(2) = frequencies_polar(maxpeakloc);
% Otherwise, search for a peak within the search window limits to determine
% initial estimates of peak edge locations.
else
    % Find the signal maximum. Values at the boundaries of the frequency
    % range are not accepted as they are not peaks. Close the search window
    % incrementally until a peak is found.
    while true
        [maxvalue,maxlocation] = max(PSD_signal(minpeakloc:maxpeakloc));
        if maxlocation == minpeakloc
            minpeakloc = minpeakloc + 1;
            continue;
        elseif maxlocation == maxpeakloc
            maxpeakloc = maxpeakloc - 1;
            continue;
        else
            maxlocation = maxlocation + minpeakloc - 1;
            break;
        end
    end

    % Find the location two sigma away from the max location.
    % At two sigma, the value of a Gaussian is e^-2 times the max value.
    % Proceed in steps of 1/10 of maximum if the two sigma location is not
    % found.
    thresholdvalue = maxvalue*exp(-2);
    thresholdstep = maxvalue*0.1;
    while true
        twosigmasleft = find(PSD_signal(1:maxlocation) < thresholdvalue,1,'last');
        if isempty(twosigmasleft)
            thresholdvalue = thresholdvalue + thresholdstep;
            continue;
        else
            break;
        end
    end
    % Find the location two sigmas right from max location.
    twosigmasright = maxlocation + (maxlocation-twosigmasleft);
    % Check validity.
    if twosigmasright == maxlocation
        twosigmasright = twosigmasright + 1;
    elseif twosigmasright > length(frequencies_polar)
        twosigmasright = length(frequencies_polar);
    end
    if twosigmasleft == maxlocation
        twosigmasleft = twosigmasleft - 1;
    elseif twosigmasleft < 1
        twosigmasleft = 1;
    end
    % Form the guesses.
    peakedge(1) = frequencies_polar(twosigmasleft);
    peakedge(2) = frequencies_polar(twosigmasright);
end

% Set lower bounds.
lower_a1 = 0;
lower_b1 = peakedge(1);
lower_c1 = (peakedge(2) - peakedge(1))/4; % Guesses should be four sigmas away from each other.
% Set higher bounds.
higher_b1 = peakedge(2);
higher_a1 = max(PSD_signal(round(length(frequencies_polar)*lower_b1/max(frequencies_polar)): ...
           round(length(frequencies_polar)*higher_b1/max(frequencies_polar))));
higher_c1 = (peakedge(2) - peakedge(1))/2; % Guesses might be just one sigma away from each other.
% Set initial values.
start_a1 = (lower_a1 + higher_a1)/2;
start_b1 = (peakedge(1) + peakedge(2))/2;
start_c1 = (lower_c1 + higher_c1)/2;
% Check bounds for validity. Upper and lower bounds should be at least
% 2e-08 away from each other to allow the computation of a
% finite-difference approximation of derivative during fitting,
% but we use 3e-08 just in case.
if higher_a1 - lower_a1 < 3e-08
    higher_a1 = lower_a1 + 3e-08;
end
if higher_b1 - lower_b1 < 3e-08
    higher_b1 = lower_b1 + 3e-08;
end
if higher_c1 - lower_c1 < 3e-08
    higher_c1 = lower_c1 + 3e-08;
end

% Set up fittype and options.
% This is a Gaussian with b1=mu and c1=std.
model = fittype( 'a1*exp(-(((x-b1)^2)/(2*c1^2)))', 'independent', 'x', 'dependent', 'y' );
opts = fitoptions( 'Method', 'NonlinearLeastSquares' );
opts.Robust = 'Off';
opts.Display = 'Off';
opts.Lower = [lower_a1 lower_b1 lower_c1];
opts.StartPoint = [start_a1 start_b1 start_c1];
opts.Upper = [higher_a1 higher_b1 higher_c1];

% Fit model to data and extract parameter values.
[fitresult, gof] = fit(frequencies_polar,PSD_signal', model, opts );
a = coeffvalues(fitresult);
peaklocation = round(length(frequencies_polar)*a(2)/max(frequencies_polar));
peakparams.a1 = a(1);
peakparams.mu1 = a(2);
peakparams.std1 = a(3);
peakparams.signalquality = 10*log10(PSD_input_1D(peaklocation)/PSD_background_1D(peaklocation));
peakparams.estimateerror = gof.rmse;

