function [cellmask] = getSegmentation(externalfilename)

% Copyright 2015 Kimmo Kartasalo
% Tampere University of Technology, Tampere, Finland
% http://www.tut.fi/cytospectre
% Email: kimmo.kartasalo@tut.fi
% 
% This file is part of CytoSpectre.
% 
% CytoSpectre is free software: you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published by
% the Free Software Foundation, either version 3 of the License, or
% (at your option) any later version.
% 
% CytoSpectre is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
% 
% You should have received a copy of the GNU General Public License
% along with CytoSpectre.  If not, see <http://www.gnu.org/licenses/>.

% Set output to empty by default.
cellmask = [];

% The mask can be of any accepted image type.
acceptedexts = {'.tif';'.TIF';'.tiff';'.TIFF';'.jpg';'.jpeg';'.JPG';'.JPEG';'.gif';'.GIF';'.bmp';'.BMP';'.png';'.PNG'};
[extpath,extfile,~] = fileparts(externalfilename);
cellmaskbasename = [extpath,filesep,'segcell_',extfile];

% Read the cell mask image.
for i=1:length(acceptedexts)
    if exist([cellmaskbasename,acceptedexts{i}],'file') > 0
        cells = imread([cellmaskbasename,acceptedexts{i}]);
        break;
    elseif i == length(acceptedexts)
        cells = [];
    end
end

% If the mask image was read successfully, process it.
if ~isempty(cells)
    % Make sure that the mask image is grayscale/binary.
    if size(cells,3) > 1
        cells = cells(:,:,1);
    end
    
    % Make sure that the mask image only contains integer values.
    cells = round(cells);

    % Find out how many different intensity values are present.
    uniquevalues = unique(cells);
    maxvalue = max(uniquevalues);
    numofvalues = length(uniquevalues);

    % If there are only two different values present, the image is binary
    % with all cells having high intensity and background having zero
    % intensity. In that case, find the connected components and label
    % them.
    if numofvalues <= 2
        cellmask = labelmatrix(bwconncomp(cells > 0));
    % If there are multiple intensity values ranging from zero to the
    % maximum value at increments of one, they are already correct labels.
    elseif numofvalues == maxvalue+1
        cellmask = cells;
    % There are multiple intensity levels, but they do not proceed 
    % at increments of one. Re-label each object with integer values
    % ranging from zero (background) to the number of intensity levels
    % (i.e. the number of objects).
    else
        cellmask = zeros(size(cells));
        for objectind = 2:numofvalues
            cellmask(cells == uniquevalues(objectind)) = objectind - 1;
        end
        
    end
end

