function [result, powerspectrum, analysispar] = mainAnalysis(inputimage, cellregion, settings)

% Copyright 2015 Kimmo Kartasalo
% Tampere University of Technology, Tampere, Finland
% http://www.tut.fi/cytospectre
% Email: kimmo.kartasalo@tut.fi
% 
% This file is part of CytoSpectre.
% 
% CytoSpectre is free software: you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published by
% the Free Software Foundation, either version 3 of the License, or
% (at your option) any later version.
% 
% CytoSpectre is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
% 
% You should have received a copy of the GNU General Public License
% along with CytoSpectre.  If not, see <http://www.gnu.org/licenses/>.

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%% PREPROCESSING %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% Get user-specified detail extraction settings or use defaults.
if settings.isdetaildefault
    analysispar.segmentlength = settings.default_segmentlength;
    analysispar.alpha = settings.default_alpha;
    analysispar.convergence_threshold = settings.default_convergence_threshold;
    analysispar.maxiterations = settings.default_maxiterations;
    analysispar.angleslack = settings.default_angleslack;
    analysispar.sigma_prior_angle = settings.default_sigma_prior_angle;
else
    analysispar.segmentlength = settings.segmentlength;
    analysispar.alpha = settings.alpha;
    analysispar.convergence_threshold = settings.convergence_threshold;
    analysispar.maxiterations = settings.maxiterations;
    analysispar.angleslack = settings.angleslack;
    analysispar.sigma_prior_angle = settings.sigma_prior_angle;
end  

% Select correct channel and convert to double.
% Image is grayscale i.e. there is only one channel.
if size(inputimage,3) == 1
    inputimage = double(inputimage);
% Image is RGB, channels are combined.
elseif size(inputimage,3) == 3 && settings.targetchannel == 4
    inputimage = sum(double(inputimage),3);
% Image is RGB, one channel is picked for analysis.
elseif size(inputimage,3) == 3 && settings.targetchannel < 4
    inputimage = double(inputimage(:,:,settings.targetchannel));
% Image is of unknown type.
else
    result = [];
    powerspectrum = [];
    analysispar = [];
    return;
end

% Get image size.
[analysispar.imsizey, analysispar.imsizex] = size(inputimage);

% If cellregion is empty i.e. the whole image is analyzed without cell
% segmentation, set cellregion equal to ones.
if isempty(cellregion)
    cellregion = ones(analysispar.imsizey,analysispar.imsizex);
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%% SPECTRAL ESTIMATION %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% Set window type, 1D to 2D window interpolation ratio and overlap.
analysispar.windowinterpolation = 10;
analysispar.overlap = [0.5 0.5];

% Set resolution parameter.
switch settings.resolutionparameter
    case 1
        analysispar.resolutionparameter = 1/5;
    case 2
        analysispar.resolutionparameter = 1/4;
    case 3
        analysispar.resolutionparameter = 1/3;
    case 4
        analysispar.resolutionparameter = 1/2;
    case 5
        analysispar.resolutionparameter = 1;
end

% Set window size according to smaller image dimension and resolution parameter.
analysispar.windowsize = round(analysispar.resolutionparameter*[min(analysispar.imsizey, analysispar.imsizex) min(analysispar.imsizey, analysispar.imsizex)]);

% Set fft size to required minimum size, or, if the window is larger than
% that, set it to the next power of two.
analysispar.nextpowsize = 2^(nextpow2(analysispar.windowsize(1)));
analysispar.minimumfftsize = 1024;
analysispar.fftlength = [max(analysispar.nextpowsize, analysispar.minimumfftsize) max(analysispar.nextpowsize, analysispar.minimumfftsize)];

% Set pixels outside the cell to zero. This only has effect if segmentation
% is enabled. Otherwise the whole image will be analyzed.
inputimage(cellregion(:) == 0) = 0;
% Estimate the power spectrum.
[powerspectrum.raw, powerspectrum.frequencies_cart, ~, analysispar.totalsegments] = estimateSpectrumWOSA(inputimage, [settings.resolution settings.resolution], analysispar);
% Check that the spectrum is valid. Faulty masks can result in failed
% spectral estimation, for example.
if any(isnan(powerspectrum.raw(:))) || any(isinf(powerspectrum.raw(:))) || any(isempty(powerspectrum.raw(:)))
    result = [];
    powerspectrum = [];
    analysispar = [];
    return;
end

% Obtain polar powerspectrum.
[powerspectrum.polar,powerspectrum.frequencies_polar,powerspectrum.theta_polar] = PSDtoPolar(powerspectrum.raw,powerspectrum.frequencies_cart,powerspectrum.frequencies_cart,pi);
% Reverse angle convention.
powerspectrum.polar = powerspectrum.polar(end:-1:1,:);

% Exclude given frequency band.
if settings.isexcludeon
    analysispar.excludelowfreq = 1/settings.excludehigh;
    analysispar.excludehighfreq = 1/settings.excludelow;
    % Check the exclusion limits.
    if analysispar.excludelowfreq < min(powerspectrum.frequencies_polar)
        analysispar.excludelowfreq = min(powerspectrum.frequencies_polar);
    end
    if analysispar.excludehighfreq > max(powerspectrum.frequencies_polar)
        analysispar.excludehighfreq = max(powerspectrum.frequencies_polar);
    end
    powerspectrum.polar = excludeBand(powerspectrum.polar,powerspectrum.frequencies_polar,analysispar.excludelowfreq,analysispar.excludehighfreq);
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%% BACKGROUND ESTIMATION %%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% Estimate background.
[powerspectrum.backgroundpolar2D] = estimateBackgroundPolar(powerspectrum.polar, powerspectrum.frequencies_polar, analysispar.segmentlength);

% Convert polar PSD to probabilistic form.
powerspectrum.polar_prob = PSDtoProb(powerspectrum.polar, powerspectrum.backgroundpolar2D, analysispar.totalsegments);

% Set lower frequency bound (i.e. higher wavelength bound).
if settings.ishighlimitauto
    % +11 corresponds to a 5 degree resolution at lowest frequency, +6
    % corresponds to a 10 degree resolution at lowest frequency.
    analysispar.lowestfreq = powerspectrum.frequencies_cart(ceil(length(powerspectrum.frequencies_cart)/2) + 11);
elseif 1/settings.highlimit < min(powerspectrum.frequencies_polar)
    analysispar.lowestfreq = min(powerspectrum.frequencies_polar);
elseif 1/settings.highlimit > max(powerspectrum.frequencies_polar)
    analysispar.lowestfreq = max(powerspectrum.frequencies_polar);
else
    analysispar.lowestfreq = 1/settings.highlimit;
end
% Set higher frequency bound (i.e. lower wavelength bound).
if settings.islowlimitauto
    analysispar.highestfreq = max(powerspectrum.frequencies_polar);
elseif 1/settings.lowlimit < min(powerspectrum.frequencies_polar)
    analysispar.highestfreq = min(powerspectrum.frequencies_polar);
elseif 1/settings.lowlimit > max(powerspectrum.frequencies_polar)
    analysispar.highestfreq = max(powerspectrum.frequencies_polar);
else
    analysispar.highestfreq = 1/settings.lowlimit;
end

% Check that upper frequency limit is higher than or equal to the lower
% frequency limit and the band includes at least one frequency bin i.e. the
% difference between upper and lower frequency limits is not smaller than
% the spacing of frequency bins.
if (analysispar.highestfreq - analysispar.lowestfreq) < (powerspectrum.frequencies_polar(2)-powerspectrum.frequencies_polar(1))
    % If the frequency range can be increased towards higher frequencies by
    % one bin, increase it. Otherwise increase it towards lower frequencies
    % by one bin.
    if (analysispar.lowestfreq + (powerspectrum.frequencies_polar(2)-powerspectrum.frequencies_polar(1))) <= max(powerspectrum.frequencies_polar)
        analysispar.highestfreq = analysispar.lowestfreq + (powerspectrum.frequencies_polar(2)-powerspectrum.frequencies_polar(1));
    else
        analysispar.lowestfreq = analysispar.highestfreq - (powerspectrum.frequencies_polar(2)-powerspectrum.frequencies_polar(1));
    end
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%% SIGNAL DETECTION %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% If detail extraction is semi-automatic, check guesses for validity.
if ~settings.isdetailauto
    analysispar.gaussianguess{1} = 1/settings.highguess;
    analysispar.gaussianguess{2} = 1/settings.lowguess;
    if analysispar.gaussianguess{1} < min(powerspectrum.frequencies_polar)
        analysispar.gaussianguess{1} = min(powerspectrum.frequencies_polar);
    end
    if analysispar.gaussianguess{2} > max(powerspectrum.frequencies_polar)
        analysispar.gaussianguess{2} = max(powerspectrum.frequencies_polar);
    end
% Otherwise set guesses to empty.    
else
    analysispar.gaussianguess = [];
    analysispar.gaussianguess{2} = [];
end

% Analyze the lowpass region to obtain principal directions.
[analysispar.maindirection, analysispar.transversedirection] = findDirections(powerspectrum.polar, powerspectrum.frequencies_polar, analysispar.lowestfreq, analysispar.highestfreq, analysispar.segmentlength, analysispar.alpha);

% Find detail component peak.
% Details are transverse to main direction.
if xor(settings.rotatemixed,settings.rotatedetail)
    [powerspectrum.PSD_1D] = getDirectedPSD(powerspectrum.polar, circ_rad2ang(powerspectrum.theta_polar), analysispar.transversedirection, analysispar.angleslack);
% Details are parallel to main direction.
else
    [powerspectrum.PSD_1D] = getDirectedPSD(powerspectrum.polar, circ_rad2ang(powerspectrum.theta_polar), analysispar.maindirection, analysispar.angleslack);
end

[analysispar.peakparams, powerspectrum.PSD_1D_signal] = findSignal(powerspectrum.PSD_1D, powerspectrum.backgroundpolar2D(1,:), powerspectrum.frequencies_polar, analysispar.lowestfreq, analysispar.highestfreq, analysispar.gaussianguess);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%% INITIAL PARAMETERS FOR PRIORS %%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Detail component prior.
analysispar.detailbandlow = analysispar.peakparams.mu1 - 2*analysispar.peakparams.std1;
if analysispar.detailbandlow < min(powerspectrum.frequencies_polar)
    analysispar.detailbandlow = min(powerspectrum.frequencies_polar);
end
analysispar.detailbandhigh = analysispar.peakparams.mu1 + 2*analysispar.peakparams.std1;
if analysispar.detailbandhigh > max(powerspectrum.frequencies_polar)
    analysispar.detailbandhigh = max(powerspectrum.frequencies_polar);
end
powerspectrum.detailband = powerspectrum.polar;
powerspectrum.detailband(:,powerspectrum.frequencies_polar < analysispar.detailbandlow) = 0;
powerspectrum.detailband(:,powerspectrum.frequencies_polar > analysispar.detailbandhigh) = 0;
analysispar.detailbandorientation = analyzeOrientations_cirstatTB(powerspectrum.detailband);
analysispar.detailbandastd = circ_rad2ang(analysispar.detailbandorientation.astd);

% Set initial parameters for detail component prior.
% Details are transverse to main direction.
if xor(settings.rotatemixed,settings.rotatedetail)
    analysispar.mu_angle_detail = analysispar.transversedirection;
% Details are parallel to main direction.
else
    analysispar.mu_angle_detail = analysispar.maindirection;
end
analysispar.mu_freq_detail = analysispar.peakparams.mu1;
analysispar.sigma_angle_detail = analysispar.sigma_prior_angle*analysispar.detailbandastd;
analysispar.sigma_freq_detail = (4*analysispar.peakparams.std1)/(2*sqrt(2*log(2))); % Gaussian Sigma = FWHM/(2*sqrt(2*log(2)).

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%% COMPONENT EXTRACTION %%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% Mixed component.
powerspectrum.ROI_mixed = powerspectrum.polar;
powerspectrum.ROI_mixed(:,powerspectrum.frequencies_polar < analysispar.lowestfreq) = 0;
powerspectrum.ROI_mixed(:,powerspectrum.frequencies_polar > analysispar.highestfreq) = 0;

% Detail component.
[powerspectrum.ROI_detail,analysispar.iter_detail] = separateSpectralROI_iterative(powerspectrum.polar, powerspectrum.polar_prob, powerspectrum.frequencies_polar, analysispar.lowestfreq, analysispar.highestfreq, analysispar.mu_angle_detail, analysispar.mu_freq_detail, analysispar.sigma_angle_detail, analysispar.sigma_freq_detail, analysispar.convergence_threshold, analysispar.maxiterations);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%% ORIENTATION ANALYSIS %%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% Mixed component.
if ~isempty(powerspectrum.ROI_mixed)
    if settings.rotatemixed
        [result.orientation_mixed] = analyzeOrientations_cirstatTB(powerspectrum.ROI_mixed,'rotate90');
    else
        [result.orientation_mixed] = analyzeOrientations_cirstatTB(powerspectrum.ROI_mixed);
    end
else
    result.orientation_mixed = [];
end

% Detail component.
if ~isempty(powerspectrum.ROI_detail)
    if settings.rotatedetail 
        [result.orientation_detail] = analyzeOrientations_cirstatTB(powerspectrum.ROI_detail,'rotate90');
    else
        [result.orientation_detail] = analyzeOrientations_cirstatTB(powerspectrum.ROI_detail);
    end
else
    result.orientation_detail = [];
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%% WAVELENGTH ANALYSIS %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% Mixed component
if ~isempty(powerspectrum.ROI_mixed)
    [result.distance_mixed, ~] = analyzeDistances(powerspectrum.ROI_mixed, powerspectrum.frequencies_polar);
else
    result.distance_mixed = [];
end

% Detail component.
if ~isempty(powerspectrum.ROI_detail)
    [result.distance_detail, ~] = analyzeDistances(powerspectrum.ROI_detail, powerspectrum.frequencies_polar);
else
    result.distance_detail = [];
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%% CONSTRUCT OUTPUT %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% Spectral regions for visualization.
mixedregion = ones(size(powerspectrum.ROI_mixed));
mixedregion(:,powerspectrum.frequencies_polar < analysispar.lowestfreq) = 0;
mixedregion(:,powerspectrum.frequencies_polar > analysispar.highestfreq) = 0;
detailregion = powerspectrum.ROI_detail > 0;
powerspectrum.mixedROIboundary = bwboundaries(mixedregion);
powerspectrum.detailROIboundary = bwboundaries(detailregion);

% Detail component SNR.
result.detailquality = 10*log10(sum(sum(powerspectrum.polar(detailregion)))/sum(sum(powerspectrum.backgroundpolar2D(detailregion))));

% Remove unnecessary variables to reduce temp file size.
powerspectrum = rmfield(powerspectrum,{'raw','frequencies_cart','backgroundpolar2D','polar_prob','detailband','ROI_mixed','ROI_detail','PSD_1D','PSD_1D_signal'});